@@ -1,5 +1,6 @@
 # orm/sync.py
-# Copyright (C) 2005-2014 the SQLAlchemy authors and contributors <see AUTHORS file>
+# Copyright (C) 2005-2015 the SQLAlchemy authors and contributors
+# <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -13,7 +14,7 @@ from . import exc, util as orm_util, attributes
 
 
 def populate(source, source_mapper, dest, dest_mapper,
-                        synchronize_pairs, uowcommit, flag_cascaded_pks):
+             synchronize_pairs, uowcommit, flag_cascaded_pks):
     source_dict = source.dict
     dest_dict = dest.dict
 
@@ -22,7 +23,7 @@ def populate(source, source_mapper, dest, dest_mapper,
             # inline of source_mapper._get_state_attr_by_column
             prop = source_mapper._columntoproperty[l]
             value = source.manager[prop.key].impl.get(source, source_dict,
-                                                    attributes.PASSIVE_OFF)
+                                                      attributes.PASSIVE_OFF)
         except exc.UnmappedColumnError:
             _raise_col_to_prop(False, source_mapper, l, dest_mapper, r)
 
@@ -39,16 +40,33 @@ def populate(source, source_mapper, dest, dest_mapper,
         # reasons, since we only need this info for a primary key
         # destination.
         if flag_cascaded_pks and l.primary_key and \
-                    r.primary_key and \
-                    r.references(l):
+                r.primary_key and \
+                r.references(l):
             uowcommit.attributes[("pk_cascaded", dest, r)] = True
 
 
+def bulk_populate_inherit_keys(
+        source_dict, source_mapper, synchronize_pairs):
+    # a simplified version of populate() used by bulk insert mode
+    for l, r in synchronize_pairs:
+        try:
+            prop = source_mapper._columntoproperty[l]
+            value = source_dict[prop.key]
+        except exc.UnmappedColumnError:
+            _raise_col_to_prop(False, source_mapper, l, source_mapper, r)
+
+        try:
+            prop = source_mapper._columntoproperty[r]
+            source_dict[prop.key] = value
+        except exc.UnmappedColumnError:
+            _raise_col_to_prop(True, source_mapper, l, source_mapper, r)
+
+
 def clear(dest, dest_mapper, synchronize_pairs):
     for l, r in synchronize_pairs:
         if r.primary_key and \
             dest_mapper._get_state_attr_by_column(
-                                        dest, dest.dict, r) is not None:
+                dest, dest.dict, r) not in orm_util._none_set:
 
             raise AssertionError(
                 "Dependency rule tried to blank-out primary key "
@@ -95,8 +113,8 @@ def source_modified(uowcommit, source, source_mapper, synchronize_pairs):
             prop = source_mapper._columntoproperty[l]
         except exc.UnmappedColumnError:
             _raise_col_to_prop(False, source_mapper, l, None, r)
-        history = uowcommit.get_attribute_history(source, prop.key,
-                                        attributes.PASSIVE_NO_INITIALIZE)
+        history = uowcommit.get_attribute_history(
+            source, prop.key, attributes.PASSIVE_NO_INITIALIZE)
         if bool(history.deleted):
             return True
     else:
@@ -106,16 +124,17 @@ def source_modified(uowcommit, source, source_mapper, synchronize_pairs):
 def _raise_col_to_prop(isdest, source_mapper, source_column,
                        dest_mapper, dest_column):
     if isdest:
-        raise exc.UnmappedColumnError("Can't execute sync rule for "
-                "destination column '%s'; mapper '%s' does not map "
-                "this column.  Try using an explicit `foreign_keys` "
-                "collection which does not include this column (or use "
-                "a viewonly=True relation)." % (dest_column,
-                dest_mapper))
+        raise exc.UnmappedColumnError(
+            "Can't execute sync rule for "
+            "destination column '%s'; mapper '%s' does not map "
+            "this column.  Try using an explicit `foreign_keys` "
+            "collection which does not include this column (or use "
+            "a viewonly=True relation)." % (dest_column, dest_mapper))
     else:
-        raise exc.UnmappedColumnError("Can't execute sync rule for "
-                "source column '%s'; mapper '%s' does not map this "
-                "column.  Try using an explicit `foreign_keys` "
-                "collection which does not include destination column "
-                "'%s' (or use a viewonly=True relation)."
-                % (source_column, source_mapper, dest_column))
+        raise exc.UnmappedColumnError(
+            "Can't execute sync rule for "
+            "source column '%s'; mapper '%s' does not map this "
+            "column.  Try using an explicit `foreign_keys` "
+            "collection which does not include destination column "
+            "'%s' (or use a viewonly=True relation)." %
+            (source_column, source_mapper, dest_column))
