@@ -1,15 +1,16 @@
-from test.lib.testing import assert_raises, assert_raises_message
+
+from sqlalchemy.testing import assert_raises, assert_raises_message
 from sqlalchemy.schema import DDL, CheckConstraint, AddConstraint, \
     DropConstraint
 from sqlalchemy import create_engine
 from sqlalchemy import MetaData, Integer, String, event, exc, text
-from test.lib.schema import Table
-from test.lib.schema import Column
+from sqlalchemy.testing.schema import Table
+from sqlalchemy.testing.schema import Column
 import sqlalchemy as tsa
-from test.lib import testing, engines
-from test.lib.testing import AssertsCompiledSQL, eq_
-from nose import SkipTest
-from test.lib import fixtures
+from sqlalchemy import testing
+from sqlalchemy.testing import engines
+from sqlalchemy.testing import AssertsCompiledSQL, eq_
+from sqlalchemy.testing import fixtures
 
 
 class DDLEventTest(fixtures.TestBase):
@@ -261,6 +262,63 @@ class DDLExecutionTest(fixtures.TestBase):
         assert 'xyzzy' in strings
         assert 'fnord' in strings
 
+    def test_deprecated_append_ddl_listener_table(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
+        canary = []
+        users.append_ddl_listener('before-create',
+                            lambda e, t, b: canary.append('mxyzptlk')
+                        )
+        users.append_ddl_listener('after-create',
+                            lambda e, t, b: canary.append('klptzyxm')
+                        )
+        users.append_ddl_listener('before-drop',
+                            lambda e, t, b: canary.append('xyzzy')
+                        )
+        users.append_ddl_listener('after-drop',
+                            lambda e, t, b: canary.append('fnord')
+                        )
+
+        metadata.create_all()
+        assert 'mxyzptlk' in canary
+        assert 'klptzyxm' in canary
+        assert 'xyzzy' not in canary
+        assert 'fnord' not in canary
+        del engine.mock[:]
+        canary[:] = []
+        metadata.drop_all()
+        assert 'mxyzptlk' not in canary
+        assert 'klptzyxm' not in canary
+        assert 'xyzzy' in canary
+        assert 'fnord' in canary
+
+    def test_deprecated_append_ddl_listener_metadata(self):
+        metadata, users, engine = self.metadata, self.users, self.engine
+        canary = []
+        metadata.append_ddl_listener('before-create',
+                            lambda e, t, b, tables=None: canary.append('mxyzptlk')
+                        )
+        metadata.append_ddl_listener('after-create',
+                            lambda e, t, b, tables=None: canary.append('klptzyxm')
+                        )
+        metadata.append_ddl_listener('before-drop',
+                            lambda e, t, b, tables=None: canary.append('xyzzy')
+                        )
+        metadata.append_ddl_listener('after-drop',
+                            lambda e, t, b, tables=None: canary.append('fnord')
+                        )
+
+        metadata.create_all()
+        assert 'mxyzptlk' in canary
+        assert 'klptzyxm' in canary
+        assert 'xyzzy' not in canary
+        assert 'fnord' not in canary
+        del engine.mock[:]
+        canary[:] = []
+        metadata.drop_all()
+        assert 'mxyzptlk' not in canary
+        assert 'klptzyxm' not in canary
+        assert 'xyzzy' in canary
+        assert 'fnord' in canary
 
     def test_metadata(self):
         metadata, engine = self.metadata, self.engine
@@ -311,8 +369,8 @@ class DDLExecutionTest(fixtures.TestBase):
         metadata, users, engine = self.metadata, self.users, self.engine
         nonpg_mock = engines.mock_engine(dialect_name='sqlite')
         pg_mock = engines.mock_engine(dialect_name='postgresql')
-        constraint = CheckConstraint('a < b', name='my_test_constraint'
-                , table=users)
+        constraint = CheckConstraint('a < b', name='my_test_constraint',
+                                    table=users)
 
         # by placing the constraint in an Add/Drop construct, the
         # 'inline_ddl' flag is set to False
@@ -347,8 +405,8 @@ class DDLExecutionTest(fixtures.TestBase):
         metadata, users, engine = self.metadata, self.users, self.engine
         nonpg_mock = engines.mock_engine(dialect_name='sqlite')
         pg_mock = engines.mock_engine(dialect_name='postgresql')
-        constraint = CheckConstraint('a < b', name='my_test_constraint'
-                , table=users)
+        constraint = CheckConstraint('a < b', name='my_test_constraint',
+                                    table=users)
 
         # by placing the constraint in an Add/Drop construct, the
         # 'inline_ddl' flag is set to False
@@ -370,11 +428,9 @@ class DDLExecutionTest(fixtures.TestBase):
         strings = ' '.join(str(x) for x in pg_mock.mock)
         assert 'my_test_constraint' in strings
 
+    @testing.requires.sqlite
     def test_ddl_execute(self):
-        try:
-            engine = create_engine('sqlite:///')
-        except ImportError:
-            raise SkipTest('Requires sqlite')
+        engine = create_engine('sqlite:///')
         cx = engine.connect()
         table = self.users
         ddl = DDL('SELECT 1')
@@ -405,28 +461,32 @@ class DDLExecutionTest(fixtures.TestBase):
                 r = eval(py)
                 assert list(r) == [(1,)], py
 
-    @testing.fails_on('postgresql+pg8000', 'pg8000 requires explicit types')
     def test_platform_escape(self):
         """test the escaping of % characters in the DDL construct."""
 
         default_from = testing.db.dialect.statement_compiler(
-                            testing.db.dialect, None).default_from()
-
-        eq_(
-            testing.db.execute(
-                text("select 'foo%something'" + default_from)
-            ).scalar(),
-            'foo%something'
-        )
-
-        eq_(
-            testing.db.execute(
-                DDL("select 'foo%%something'" + default_from)
-            ).scalar(),
-            'foo%something'
-        )
+            testing.db.dialect, None).default_from()
 
+        # We're abusing the DDL()
+        # construct here by pushing a SELECT through it
+        # so that we can verify the round trip.
+        # the DDL() will trigger autocommit, which prohibits
+        # some DBAPIs from returning results (pyodbc), so we
+        # run in an explicit transaction.
+        with testing.db.begin() as conn:
+            eq_(
+                conn.execute(
+                    text("select 'foo%something'" + default_from)
+                ).scalar(),
+                'foo%something'
+            )
 
+            eq_(
+                conn.execute(
+                    DDL("select 'foo%%something'" + default_from)
+                ).scalar(),
+                'foo%something'
+            )
 
 
 class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
@@ -465,12 +525,11 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
                             dialect=dialect)
         self.assert_compile(ddl.against(sane_schema), 'S S-T T-s.t-b',
                             dialect=dialect)
-        self.assert_compile(ddl.against(insane_alone), 'S S-T T-"t t"-b'
-                            , dialect=dialect)
+        self.assert_compile(ddl.against(insane_alone), 'S S-T T-"t t"-b',
+                            dialect=dialect)
         self.assert_compile(ddl.against(insane_schema),
                             'S S-T T-"s s"."t t"-b', dialect=dialect)
 
-
     def test_filter(self):
         cx = self.mock_engine()
 
@@ -481,10 +540,10 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
         assert DDL('').execute_if(dialect=target)._should_execute(tbl, cx)
         assert not DDL('').execute_if(dialect='bogus').\
                         _should_execute(tbl, cx)
-        assert DDL('').execute_if(callable_=lambda d, y,z, **kw: True).\
+        assert DDL('').execute_if(callable_=lambda d, y, z, **kw: True).\
                         _should_execute(tbl, cx)
         assert(DDL('').execute_if(
-                        callable_=lambda d, y,z, **kw: z.engine.name 
+                        callable_=lambda d, y, z, **kw: z.engine.name
                         != 'bogus').
                _should_execute(tbl, cx))
 
@@ -499,16 +558,14 @@ class DDLTest(fixtures.TestBase, AssertsCompiledSQL):
         assert DDL('', on=target)._should_execute_deprecated('x', tbl, cx)
         assert not DDL('', on='bogus').\
                         _should_execute_deprecated('x', tbl, cx)
-        assert DDL('', on=lambda d, x,y,z: True).\
+        assert DDL('', on=lambda d, x, y, z: True).\
                         _should_execute_deprecated('x', tbl, cx)
-        assert(DDL('', on=lambda d, x,y,z: z.engine.name != 'bogus').
+        assert(DDL('', on=lambda d, x, y, z: z.engine.name != 'bogus').
                _should_execute_deprecated('x', tbl, cx))
 
     def test_repr(self):
         assert repr(DDL('s'))
         assert repr(DDL('s', on='engine'))
         assert repr(DDL('s', on=lambda x: 1))
-        assert repr(DDL('s', context={'a':1}))
-        assert repr(DDL('s', on='engine', context={'a':1}))
-
-
+        assert repr(DDL('s', context={'a': 1}))
+        assert repr(DDL('s', on='engine', context={'a': 1}))
