@@ -1,14 +1,15 @@
-from sqlalchemy.testing import assert_raises, assert_raises_message, eq_
-import ConfigParser
-import StringIO
+from sqlalchemy.testing import assert_raises, eq_, assert_raises_message
 import sqlalchemy.engine.url as url
 from sqlalchemy import create_engine, engine_from_config, exc, pool
-from sqlalchemy.engine.util import _coerce_config
 from sqlalchemy.engine.default import DefaultDialect
 import sqlalchemy as tsa
 from sqlalchemy.testing import fixtures
 from sqlalchemy import testing
-from sqlalchemy.testing.mock import Mock
+from sqlalchemy.testing.mock import Mock, MagicMock
+from sqlalchemy import event
+from sqlalchemy import select
+
+dialect = None
 
 
 class ParseConnectTest(fixtures.TestBase):
@@ -33,40 +34,70 @@ class ParseConnectTest(fixtures.TestBase):
             'dbtype://username:password@/database',
             'dbtype:////usr/local/_xtest@example.com/members.db',
             'dbtype://username:apples%2Foranges@hostspec/database',
-            'dbtype://username:password@[2001:da8:2004:1000:202:116:160:90]/database?foo=bar',
-            'dbtype://username:password@[2001:da8:2004:1000:202:116:160:90]:80/database?foo=bar'
-            ):
+            'dbtype://username:password@[2001:da8:2004:1000:202:116:160:90]'
+            '/database?foo=bar',
+            'dbtype://username:password@[2001:da8:2004:1000:202:116:160:90]:80'
+            '/database?foo=bar'
+        ):
             u = url.make_url(text)
 
             assert u.drivername in ('dbtype', 'dbtype+apitype')
             assert u.username in ('username', None)
             assert u.password in ('password', 'apples/oranges', None)
-            assert u.host in ('hostspec', '127.0.0.1',
-                        '2001:da8:2004:1000:202:116:160:90', '', None), u.host
-            assert u.database in ('database',
-                    '/usr/local/_xtest@example.com/members.db',
-                    '/usr/db_file.db', ':memory:', '',
-                    'foo/bar/im/a/file',
-                    'E:/work/src/LEM/db/hello.db', None), u.database
+            assert u.host in (
+                'hostspec', '127.0.0.1',
+                '2001:da8:2004:1000:202:116:160:90', '', None), u.host
+            assert u.database in (
+                'database',
+                '/usr/local/_xtest@example.com/members.db',
+                '/usr/db_file.db', ':memory:', '',
+                'foo/bar/im/a/file',
+                'E:/work/src/LEM/db/hello.db', None), u.database
             eq_(str(u), text)
 
+    def test_rfc1738_password(self):
+        u = url.make_url("dbtype://user:pass word + other%3Awords@host/dbname")
+        eq_(u.password, "pass word + other:words")
+        eq_(str(u), "dbtype://user:pass word + other%3Awords@host/dbname")
+
+        u = url.make_url(
+            'dbtype://username:apples%2Foranges@hostspec/database')
+        eq_(u.password, "apples/oranges")
+        eq_(str(u), 'dbtype://username:apples%2Foranges@hostspec/database')
+
+        u = url.make_url(
+            'dbtype://username:apples%40oranges%40%40@hostspec/database')
+        eq_(u.password, "apples@oranges@@")
+        eq_(
+            str(u),
+            'dbtype://username:apples%40oranges%40%40@hostspec/database')
+
+        u = url.make_url('dbtype://username%40:@hostspec/database')
+        eq_(u.password, '')
+        eq_(u.username, "username@")
+        eq_(str(u), 'dbtype://username%40:@hostspec/database')
+
+        u = url.make_url('dbtype://username:pass%2Fword@hostspec/database')
+        eq_(u.password, 'pass/word')
+        eq_(str(u), 'dbtype://username:pass%2Fword@hostspec/database')
+
+
 class DialectImportTest(fixtures.TestBase):
     def test_import_base_dialects(self):
-
         # the globals() somehow makes it for the exec() + nose3.
 
         for name in (
-            'mysql',
-            'firebird',
-            'postgresql',
-            'sqlite',
-            'oracle',
-            'mssql',
-            ):
+                'mysql',
+                'firebird',
+                'postgresql',
+                'sqlite',
+                'oracle',
+                'mssql'):
             exec ('from sqlalchemy.dialects import %s\ndialect = '
                   '%s.dialect()' % (name, name), globals())
             eq_(dialect.name, name)
 
+
 class CreateEngineTest(fixtures.TestBase):
     """test that create_engine arguments of different types get
     propagated properly"""
@@ -77,70 +108,28 @@ class CreateEngineTest(fixtures.TestBase):
             create_engine('postgresql://scott:tiger@somehost/test?foobe'
                           'r=12&lala=18&fooz=somevalue', module=dbapi,
                           _initialize=False)
-        c = e.connect()
+        e.connect()
 
     def test_kwargs(self):
         dbapi = MockDBAPI(foober=12, lala=18, hoho={'this': 'dict'},
                           fooz='somevalue')
         e = \
-            create_engine('postgresql://scott:tiger@somehost/test?fooz='
-                          'somevalue', connect_args={'foober': 12,
-                          'lala': 18, 'hoho': {'this': 'dict'}},
-                          module=dbapi, _initialize=False)
-        c = e.connect()
-
-    def test_coerce_config(self):
-        raw = r"""
-[prefixed]
-sqlalchemy.url=postgresql://scott:tiger@somehost/test?fooz=somevalue
-sqlalchemy.convert_unicode=0
-sqlalchemy.echo=false
-sqlalchemy.echo_pool=1
-sqlalchemy.max_overflow=2
-sqlalchemy.pool_recycle=50
-sqlalchemy.pool_size=2
-sqlalchemy.pool_threadlocal=1
-sqlalchemy.pool_timeout=10
-[plain]
-url=postgresql://scott:tiger@somehost/test?fooz=somevalue
-convert_unicode=0
-echo=0
-echo_pool=1
-max_overflow=2
-pool_recycle=50
-pool_size=2
-pool_threadlocal=1
-pool_timeout=10
-"""
-        ini = ConfigParser.ConfigParser()
-        ini.readfp(StringIO.StringIO(raw))
-
-        expected = {
-            'url': 'postgresql://scott:tiger@somehost/test?fooz=somevalue',
-            'convert_unicode': 0,
-            'echo': False,
-            'echo_pool': True,
-            'max_overflow': 2,
-            'pool_recycle': 50,
-            'pool_size': 2,
-            'pool_threadlocal': True,
-            'pool_timeout': 10,
-            }
-
-        prefixed = dict(ini.items('prefixed'))
-        self.assert_(_coerce_config(prefixed, 'sqlalchemy.')
-                     == expected)
-
-        plain = dict(ini.items('plain'))
-        self.assert_(_coerce_config(plain, '') == expected)
+            create_engine(
+                'postgresql://scott:tiger@somehost/test?fooz='
+                'somevalue', connect_args={
+                    'foober': 12,
+                    'lala': 18, 'hoho': {'this': 'dict'}},
+                module=dbapi, _initialize=False)
+        e.connect()
 
     def test_engine_from_config(self):
         dbapi = mock_dbapi
 
-        config = \
-            {'sqlalchemy.url': 'postgresql://scott:tiger@somehost/test'\
-             '?fooz=somevalue', 'sqlalchemy.pool_recycle': '50',
-             'sqlalchemy.echo': 'true'}
+        config = {
+            'sqlalchemy.url': 'postgresql://scott:tiger@somehost/test'
+            '?fooz=somevalue',
+            'sqlalchemy.pool_recycle': '50',
+            'sqlalchemy.echo': 'true'}
 
         e = engine_from_config(config, module=dbapi, _initialize=False)
         assert e.pool._recycle == 50
@@ -149,20 +138,35 @@ pool_timeout=10
                             'z=somevalue')
         assert e.echo is True
 
-        for param, values in [
-            ('convert_unicode', ('true', 'false', 'force')),
-            ('echo', ('true', 'false', 'debug')),
-            ('echo_pool', ('true', 'false', 'debug')),
-            ('use_native_unicode', ('true', 'false')),
-        ]:
-            for value in values:
-                config = {
-                        'sqlalchemy.url': 'postgresql://scott:tiger@somehost/test',
-                        'sqlalchemy.%s' % param : value
-                }
-                cfg = _coerce_config(config, 'sqlalchemy.')
-                assert cfg[param] == {'true':True, 'false':False}.get(value, value)
+    def test_engine_from_config_custom(self):
+        from sqlalchemy import util
+        from sqlalchemy.dialects import registry
+        tokens = __name__.split(".")
 
+        class MyDialect(MockDialect):
+            engine_config_types = {
+                "foobar": int,
+                "bathoho": util.bool_or_str('force')
+            }
+
+            def __init__(self, foobar=None, bathoho=None, **kw):
+                self.foobar = foobar
+                self.bathoho = bathoho
+
+        global dialect
+        dialect = MyDialect
+        registry.register(
+            "mockdialect.barb",
+            ".".join(tokens[0:-1]), tokens[-1])
+
+        config = {
+            "sqlalchemy.url": "mockdialect+barb://",
+            "sqlalchemy.foobar": "5",
+            "sqlalchemy.bathoho": "false"
+        }
+        e = engine_from_config(config, _initialize=False)
+        eq_(e.dialect.foobar, 5)
+        eq_(e.dialect.bathoho, False)
 
     def test_custom(self):
         dbapi = MockDBAPI(foober=12, lala=18, hoho={'this': 'dict'},
@@ -177,7 +181,7 @@ pool_timeout=10
 
         e = create_engine('postgresql://', creator=connect,
                           module=dbapi, _initialize=False)
-        c = e.connect()
+        e.connect()
 
     def test_recycle(self):
         dbapi = MockDBAPI(foober=12, lala=18, hoho={'this': 'dict'},
@@ -196,8 +200,9 @@ pool_timeout=10
             (True, pool.reset_rollback),
             (False, pool.reset_none),
         ]:
-            e = create_engine('postgresql://', pool_reset_on_return=value,
-                          module=dbapi, _initialize=False)
+            e = create_engine(
+                'postgresql://', pool_reset_on_return=value,
+                module=dbapi, _initialize=False)
             assert e.pool._reset_on_return is expected
 
         assert_raises(
@@ -225,7 +230,7 @@ pool_timeout=10
             lala=5,
             use_ansi=True,
             module=mock_dbapi,
-            )
+        )
         assert_raises(TypeError, create_engine, 'postgresql://',
                       lala=5, module=mock_dbapi)
         assert_raises(TypeError, create_engine, 'sqlite://', lala=5,
@@ -233,48 +238,6 @@ pool_timeout=10
         assert_raises(TypeError, create_engine, 'mysql+mysqldb://',
                       use_unicode=True, module=mock_dbapi)
 
-    @testing.requires.sqlite
-    def test_wraps_connect_in_dbapi(self):
-        # sqlite uses SingletonThreadPool which doesnt have max_overflow
-
-        assert_raises(TypeError, create_engine, 'sqlite://',
-                      max_overflow=5, module=mock_sqlite_dbapi)
-        e = create_engine('sqlite://', connect_args={'use_unicode'
-                          : True}, convert_unicode=True)
-        try:
-            e.connect()
-        except tsa.exc.DBAPIError, de:
-            assert not de.connection_invalidated
-
-    def test_ensure_dialect_does_is_disconnect_no_conn(self):
-        """test that is_disconnect() doesn't choke if no connection, cursor given."""
-        dialect = testing.db.dialect
-        dbapi = dialect.dbapi
-        assert not dialect.is_disconnect(dbapi.OperationalError("test"), None, None)
-
-    @testing.requires.sqlite
-    def test_invalidate_on_connect(self):
-        """test that is_disconnect() is called during connect.
-
-        interpretation of connection failures are not supported by
-        every backend.
-
-        """
-
-        e = create_engine('sqlite://')
-        sqlite3 = e.dialect.dbapi
-
-        dbapi = MockDBAPI()
-        dbapi.Error = sqlite3.Error,
-        dbapi.ProgrammingError = sqlite3.ProgrammingError
-        dbapi.connect = Mock(side_effect=sqlite3.ProgrammingError(
-                                    "Cannot operate on a closed database."))
-        try:
-            create_engine('sqlite://', module=dbapi).connect()
-            assert False
-        except tsa.exc.DBAPIError, de:
-            assert de.connection_invalidated
-
     def test_urlattr(self):
         """test the url attribute on ``Engine``."""
 
@@ -300,7 +263,7 @@ pool_timeout=10
             echo_pool=None,
             module=mock_dbapi,
             _initialize=False,
-            )
+        )
         assert e.pool._recycle == 50
 
         # these args work for QueuePool
@@ -312,7 +275,7 @@ pool_timeout=10
             poolclass=tsa.pool.QueuePool,
             module=mock_dbapi,
             _initialize=False,
-            )
+        )
 
         # but not SingletonThreadPool
 
@@ -325,7 +288,8 @@ pool_timeout=10
             poolclass=tsa.pool.SingletonThreadPool,
             module=mock_sqlite_dbapi,
             _initialize=False,
-            )
+        )
+
 
 class TestRegNewDBAPI(fixtures.TestBase):
     def test_register_base(self):
@@ -348,7 +312,8 @@ class TestRegNewDBAPI(fixtures.TestBase):
 
         global dialect
         dialect = MockDialect
-        registry.register("mockdialect.foob", ".".join(tokens[0:-1]), tokens[-1])
+        registry.register(
+            "mockdialect.foob", ".".join(tokens[0:-1]), tokens[-1])
 
         e = create_engine("mockdialect+foob://")
         assert isinstance(e.dialect, MockDialect)
@@ -360,13 +325,16 @@ class TestRegNewDBAPI(fixtures.TestBase):
         e = create_engine("mysql+my_mock_dialect://")
         assert isinstance(e.dialect, MockDialect)
 
+
 class MockDialect(DefaultDialect):
     @classmethod
     def dbapi(cls, **kw):
         return MockDBAPI()
 
+
 def MockDBAPI(**assert_kwargs):
     connection = Mock(get_server_version_info=Mock(return_value='5.0'))
+
     def connect(*args, **kwargs):
         for k in assert_kwargs:
             assert k in kwargs, 'key %s not present in dictionary' % k
@@ -375,13 +343,13 @@ def MockDBAPI(**assert_kwargs):
             )
         return connection
 
-    return Mock(
-                sqlite_version_info=(99, 9, 9,),
-                version_info=(99, 9, 9,),
-                sqlite_version='99.9.9',
-                paramstyle='named',
-                connect=Mock(side_effect=connect)
-            )
+    return MagicMock(
+        sqlite_version_info=(99, 9, 9,),
+        version_info=(99, 9, 9,),
+        sqlite_version='99.9.9',
+        paramstyle='named',
+        connect=Mock(side_effect=connect)
+    )
 
 mock_dbapi = MockDBAPI()
 mock_sqlite_dbapi = msd = MockDBAPI()
