@@ -3,19 +3,21 @@ from sqlalchemy.testing import eq_, assert_raises, \
 from sqlalchemy.ext import declarative as decl
 import sqlalchemy as sa
 from sqlalchemy import testing
-from sqlalchemy import Integer, String, ForeignKey
+from sqlalchemy import Integer, String, ForeignKey, select, func
 from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import relationship, create_session, class_mapper, \
     configure_mappers, clear_mappers, \
-    deferred, column_property, \
-    Session
+    deferred, column_property, Session, base as orm_base
 from sqlalchemy.util import classproperty
 from sqlalchemy.ext.declarative import declared_attr
-from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import fixtures, mock
+from sqlalchemy.testing.util import gc_collect
 
 Base = None
 
+
 class DeclarativeTestBase(fixtures.TestBase, testing.AssertsExecutionResults):
+
     def setup(self):
         global Base
         Base = decl.declarative_base(testing.db)
@@ -25,6 +27,7 @@ class DeclarativeTestBase(fixtures.TestBase, testing.AssertsExecutionResults):
         clear_mappers()
         Base.metadata.drop_all()
 
+
 class DeclarativeMixinTest(DeclarativeTestBase):
 
     def test_simple(self):
@@ -115,7 +118,6 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         eq_(MyModelA.__table__.c.foo.type.__class__, String)
         eq_(MyModelB.__table__.c.foo.type.__class__, Integer)
 
-
     def test_not_allowed(self):
 
         class MyMixin:
@@ -158,6 +160,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_table_name_inherited(self):
 
         class MyMixin:
+
             @declared_attr
             def __tablename__(cls):
                 return cls.__name__.lower()
@@ -170,6 +173,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
     def test_classproperty_still_works(self):
         class MyMixin(object):
+
             @classproperty
             def __tablename__(cls):
                 return cls.__name__.lower()
@@ -183,6 +187,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_table_name_not_inherited(self):
 
         class MyMixin:
+
             @declared_attr
             def __tablename__(cls):
                 return cls.__name__.lower()
@@ -196,11 +201,13 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_table_name_inheritance_order(self):
 
         class MyMixin1:
+
             @declared_attr
             def __tablename__(cls):
                 return cls.__name__.lower() + '1'
 
         class MyMixin2:
+
             @declared_attr
             def __tablename__(cls):
                 return cls.__name__.lower() + '2'
@@ -213,6 +220,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_table_name_dependent_on_subclass(self):
 
         class MyHistoryMixin:
+
             @declared_attr
             def __tablename__(cls):
                 return cls.parent_name + '_changelog'
@@ -237,6 +245,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_table_args_inherited_descriptor(self):
 
         class MyMixin:
+
             @declared_attr
             def __table_args__(cls):
                 return {'info': cls.__name__}
@@ -290,7 +299,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         assert Specific.bar.prop is General.bar.prop
 
     @testing.skip_if(lambda: testing.against('oracle'),
-                    "Test has an empty insert in it at the moment")
+                     "Test has an empty insert in it at the moment")
     def test_columns_single_inheritance_conflict_resolution(self):
         """Test that a declared_attr can return the existing column and it will
         be ignored.  this allows conditional columns to be added.
@@ -303,20 +312,24 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             id = Column(Integer, primary_key=True)
 
         class Mixin(object):
+
             @declared_attr
             def target_id(cls):
-                return cls.__table__.c.get('target_id',
-                        Column(Integer, ForeignKey('other.id'))
-                    )
+                return cls.__table__.c.get(
+                    'target_id',
+                    Column(Integer, ForeignKey('other.id'))
+                )
 
             @declared_attr
             def target(cls):
                 return relationship("Other")
 
         class Engineer(Mixin, Person):
+
             """single table inheritance"""
 
         class Manager(Mixin, Person):
+
             """single table inheritance"""
 
         class Other(Base):
@@ -339,11 +352,10 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             Engineer(target=o1),
             Manager(target=o2),
             Manager(target=o1)
-            ])
+        ])
         session.commit()
         eq_(session.query(Engineer).first().target, o1)
 
-
     def test_columns_joined_table_inheritance(self):
         """Test a column on a mixin with an alternate attribute name,
         mapped to a superclass and joined-table inheritance subclass.
@@ -429,6 +441,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
     def test_mapper_args_declared_attr(self):
 
         class ComputedMapperArgs:
+
             @declared_attr
             def __mapper_args__(cls):
                 if cls.__name__ == 'Person':
@@ -455,6 +468,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         # ComputedMapperArgs on both classes for no apparent reason.
 
         class ComputedMapperArgs:
+
             @declared_attr
             def __mapper_args__(cls):
                 if cls.__name__ == 'Person':
@@ -613,7 +627,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
             @declared_attr
             def __table_args__(cls):
-                return {'mysql_engine':'InnoDB'}
+                return {'mysql_engine': 'InnoDB'}
 
             @declared_attr
             def __mapper_args__(cls):
@@ -641,13 +655,14 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         """test the @declared_attr approach from a custom base."""
 
         class Base(object):
+
             @declared_attr
             def __tablename__(cls):
                 return cls.__name__.lower()
 
             @declared_attr
             def __table_args__(cls):
-                return {'mysql_engine':'InnoDB'}
+                return {'mysql_engine': 'InnoDB'}
 
             @declared_attr
             def id(self):
@@ -715,7 +730,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         eq_(Generic.__table__.name, 'generic')
         eq_(Specific.__table__.name, 'specific')
         eq_(list(Generic.__table__.c.keys()), ['timestamp', 'id',
-            'python_type'])
+                                               'python_type'])
         eq_(list(Specific.__table__.c.keys()), ['id'])
         eq_(Generic.__table__.kwargs, {'mysql_engine': 'InnoDB'})
         eq_(Specific.__table__.kwargs, {'mysql_engine': 'InnoDB'})
@@ -750,7 +765,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
         eq_(BaseType.__table__.name, 'basetype')
         eq_(list(BaseType.__table__.c.keys()), ['timestamp', 'type', 'id',
-            'value'])
+                                                'value'])
         eq_(BaseType.__table__.kwargs, {'mysql_engine': 'InnoDB'})
         assert Single.__table__ is BaseType.__table__
         eq_(Joined.__table__.name, 'joined')
@@ -852,7 +867,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             @declared_attr
             def __tablename__(cls):
                 if decl.has_inherited_table(cls) and TableNameMixin \
-                    not in cls.__bases__:
+                        not in cls.__bases__:
                     return None
                 return cls.__name__.lower()
 
@@ -901,9 +916,9 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
         class Model(Base, ColumnMixin):
 
-            __table__ = Table('foo', Base.metadata, Column('data',
-                              Integer), Column('id', Integer,
-                              primary_key=True))
+            __table__ = Table('foo', Base.metadata,
+                              Column('data', Integer),
+                              Column('id', Integer, primary_key=True))
 
         model_col = Model.__table__.c.data
         mixin_col = ColumnMixin.data
@@ -921,8 +936,8 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             class Model(Base, ColumnMixin):
 
                 __table__ = Table('foo', Base.metadata,
-                                Column('data',Integer),
-                                Column('id', Integer,primary_key=True))
+                                  Column('data', Integer),
+                                  Column('id', Integer, primary_key=True))
                 foo = relationship("Dest")
 
         assert_raises_message(sa.exc.ArgumentError,
@@ -943,9 +958,9 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             class Model(Base, ColumnMixin):
 
                 __table__ = Table('foo', Base.metadata,
-                                Column('data',Integer),
-                                Column('tada', Integer),
-                                Column('id', Integer,primary_key=True))
+                                  Column('data', Integer),
+                                  Column('tada', Integer),
+                                  Column('id', Integer, primary_key=True))
                 foo = relationship("Dest")
 
         assert_raises_message(sa.exc.ArgumentError,
@@ -960,9 +975,9 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
         class Model(Base, ColumnMixin):
 
-            __table__ = Table('foo', Base.metadata, Column('data',
-                              Integer), Column('id', Integer,
-                              primary_key=True))
+            __table__ = Table('foo', Base.metadata,
+                              Column('data', Integer),
+                              Column('id', Integer, primary_key=True))
 
         model_col = Model.__table__.c.data
         mixin_col = ColumnMixin.data
@@ -988,10 +1003,11 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             __tablename__ = 'model'
 
         eq_(list(Model.__table__.c.keys()), ['col1', 'col3', 'col2', 'col4',
-            'id'])
+                                             'id'])
 
     def test_honor_class_mro_one(self):
         class HasXMixin(object):
+
             @declared_attr
             def x(self):
                 return Column(Integer)
@@ -1008,6 +1024,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
     def test_honor_class_mro_two(self):
         class HasXMixin(object):
+
             @declared_attr
             def x(self):
                 return Column(Integer)
@@ -1015,6 +1032,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         class Parent(HasXMixin, Base):
             __tablename__ = 'parent'
             id = Column(Integer, primary_key=True)
+
             def x(self):
                 return "hi"
 
@@ -1026,6 +1044,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
 
     def test_arbitrary_attrs_one(self):
         class HasMixin(object):
+
             @declared_attr
             def some_attr(cls):
                 return cls.__name__ + "SOME ATTR"
@@ -1044,8 +1063,9 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             __tablename__ = 'filter_a'
             id = Column(Integer(), primary_key=True)
             parent_id = Column(Integer(),
-                    ForeignKey('type_a.id'))
+                               ForeignKey('type_a.id'))
             filter = Column(String())
+
             def __init__(self, filter_, **kw):
                 self.filter = filter_
 
@@ -1053,16 +1073,18 @@ class DeclarativeMixinTest(DeclarativeTestBase):
             __tablename__ = 'filter_b'
             id = Column(Integer(), primary_key=True)
             parent_id = Column(Integer(),
-                    ForeignKey('type_b.id'))
+                               ForeignKey('type_b.id'))
             filter = Column(String())
+
             def __init__(self, filter_, **kw):
                 self.filter = filter_
 
         class FilterMixin(object):
+
             @declared_attr
             def _filters(cls):
                 return relationship(cls.filter_class,
-                        cascade='all,delete,delete-orphan')
+                                    cascade='all,delete,delete-orphan')
 
             @declared_attr
             def filters(cls):
@@ -1081,6 +1103,7 @@ class DeclarativeMixinTest(DeclarativeTestBase):
         TypeA(filters=['foo'])
         TypeB(filters=['foo'])
 
+
 class DeclarativeMixinPropertyTest(DeclarativeTestBase):
 
     def test_column_property(self):
@@ -1119,9 +1142,9 @@ class DeclarativeMixinPropertyTest(DeclarativeTestBase):
         sess.add_all([m1, m2])
         sess.flush()
         eq_(sess.query(MyModel).filter(MyModel.prop_hoho == 'foo'
-            ).one(), m1)
+                                       ).one(), m1)
         eq_(sess.query(MyOtherModel).filter(MyOtherModel.prop_hoho
-            == 'bar').one(), m2)
+                                            == 'bar').one(), m2)
 
     def test_doc(self):
         """test documentation transfer.
@@ -1199,7 +1222,6 @@ class DeclarativeMixinPropertyTest(DeclarativeTestBase):
             ModelTwo.__table__.c.version_id
         )
 
-
     def test_deferred(self):
 
         class MyMixin(object):
@@ -1236,8 +1258,8 @@ class DeclarativeMixinPropertyTest(DeclarativeTestBase):
                 @declared_attr
                 def target(cls):
                     return relationship('Target',
-                            primaryjoin='Target.id==%s.target_id'
-                            % cls.__name__)
+                                        primaryjoin='Target.id==%s.target_id'
+                                        % cls.__name__)
             else:
 
                 @declared_attr
@@ -1279,3 +1301,346 @@ class DeclarativeMixinPropertyTest(DeclarativeTestBase):
     def test_relationship_primryjoin(self):
         self._test_relationship(True)
 
+
+class DeclaredAttrTest(DeclarativeTestBase, testing.AssertsCompiledSQL):
+    __dialect__ = 'default'
+
+    def test_singleton_behavior_within_decl(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr
+            def my_prop(cls):
+                counter(cls)
+                return Column('x', Integer)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def my_other_prop(cls):
+                return column_property(cls.my_prop + 5)
+
+        eq_(counter.mock_calls, [mock.call(A)])
+
+        class B(Base, Mixin):
+            __tablename__ = 'b'
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def my_other_prop(cls):
+                return column_property(cls.my_prop + 5)
+
+        eq_(
+            counter.mock_calls,
+            [mock.call(A), mock.call(B)])
+
+        # this is why we need singleton-per-class behavior.   We get
+        # an un-bound "x" column otherwise here, because my_prop() generates
+        # multiple columns.
+        a_col = A.my_other_prop.__clause_element__().element.left
+        b_col = B.my_other_prop.__clause_element__().element.left
+        is_(a_col.table, A.__table__)
+        is_(b_col.table, B.__table__)
+        is_(a_col, A.__table__.c.x)
+        is_(b_col, B.__table__.c.x)
+
+        s = Session()
+        self.assert_compile(
+            s.query(A),
+            "SELECT a.x AS a_x, a.x + :x_1 AS anon_1, a.id AS a_id FROM a"
+        )
+        self.assert_compile(
+            s.query(B),
+            "SELECT b.x AS b_x, b.x + :x_1 AS anon_1, b.id AS b_id FROM b"
+        )
+
+
+    def test_singleton_gc(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr
+            def my_prop(cls):
+                counter(cls.__name__)
+                return Column('x', Integer)
+
+        class A(Base, Mixin):
+            __tablename__ = 'b'
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def my_other_prop(cls):
+                return column_property(cls.my_prop + 5)
+
+        eq_(counter.mock_calls, [mock.call("A")])
+        del A
+        gc_collect()
+        assert "A" not in Base._decl_class_registry
+
+    def test_can_we_access_the_mixin_straight(self):
+        class Mixin(object):
+            @declared_attr
+            def my_prop(cls):
+                return Column('x', Integer)
+
+        assert_raises_message(
+            sa.exc.SAWarning,
+            "Unmanaged access of declarative attribute my_prop "
+            "from non-mapped class Mixin",
+            getattr, Mixin, "my_prop"
+        )
+
+    def test_non_decl_access(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr
+            def __tablename__(cls):
+                counter(cls)
+                return "foo"
+
+        class Foo(Mixin, Base):
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def x(cls):
+                cls.__tablename__
+
+            @declared_attr
+            def y(cls):
+                cls.__tablename__
+
+        eq_(
+            counter.mock_calls,
+            [mock.call(Foo)]
+        )
+
+        eq_(Foo.__tablename__, 'foo')
+        eq_(Foo.__tablename__, 'foo')
+
+        eq_(
+            counter.mock_calls,
+            [mock.call(Foo), mock.call(Foo), mock.call(Foo)]
+        )
+
+    def test_property_noncascade(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        class B(A):
+            pass
+
+        eq_(counter.mock_calls, [mock.call(A)])
+
+    def test_property_cascade(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr.cascading
+            def my_prop(cls):
+                counter(cls)
+                return column_property(cls.x + 2)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            x = Column(Integer)
+
+        class B(A):
+            pass
+
+        eq_(counter.mock_calls, [mock.call(A), mock.call(B)])
+
+    def test_col_prop_attrs_associated_w_class_for_mapper_args(self):
+        from sqlalchemy import Column
+        import collections
+
+        asserted = collections.defaultdict(set)
+
+        class Mixin(object):
+            @declared_attr.cascading
+            def my_attr(cls):
+                if decl.has_inherited_table(cls):
+                    id = Column(ForeignKey('a.my_attr'), primary_key=True)
+                    asserted['b'].add(id)
+                else:
+                    id = Column(Integer, primary_key=True)
+                    asserted['a'].add(id)
+                return id
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            @declared_attr
+            def __mapper_args__(cls):
+                asserted['a'].add(cls.my_attr)
+                return {}
+
+        # here:
+        # 1. A is mapped.  so A.my_attr is now the InstrumentedAttribute.
+        # 2. B wants to call my_attr also.  Due to .cascading, it has been
+        # invoked specific to B, and is present in the dict_ that will
+        # be used when we map the class.  But except for the
+        # special setattr() we do in _scan_attributes() in this case, would
+        # otherwise not been set on the class as anything from this call;
+        # the usual mechanics of calling it from the descriptor also do not
+        # work because A is fully mapped and because A set it up, is currently
+        # that non-expected InstrumentedAttribute and replaces the
+        # descriptor from being invoked.
+
+        class B(A):
+            __tablename__ = 'b'
+
+            @declared_attr
+            def __mapper_args__(cls):
+                asserted['b'].add(cls.my_attr)
+                return {}
+
+        eq_(
+            asserted,
+            {
+                'a': set([A.my_attr.property.columns[0]]),
+                'b': set([B.my_attr.property.columns[0]])
+            }
+        )
+
+    def test_column_pre_map(self):
+        counter = mock.Mock()
+
+        class Mixin(object):
+            @declared_attr
+            def my_col(cls):
+                counter(cls)
+                assert not orm_base._mapper_or_none(cls)
+                return Column('x', Integer)
+
+        class A(Base, Mixin):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+
+        eq_(counter.mock_calls, [mock.call(A)])
+
+    def test_mixin_attr_refers_to_column_copies(self):
+        # this @declared_attr can refer to User.id
+        # freely because we now do the "copy column" operation
+        # before the declared_attr is invoked.
+
+        counter = mock.Mock()
+
+        class HasAddressCount(object):
+            id = Column(Integer, primary_key=True)
+
+            @declared_attr
+            def address_count(cls):
+                counter(cls.id)
+                return column_property(
+                    select([func.count(Address.id)]).
+                    where(Address.user_id == cls.id).
+                    as_scalar()
+                )
+
+        class Address(Base):
+            __tablename__ = 'address'
+            id = Column(Integer, primary_key=True)
+            user_id = Column(ForeignKey('user.id'))
+
+        class User(Base, HasAddressCount):
+            __tablename__ = 'user'
+
+        eq_(
+            counter.mock_calls,
+            [mock.call(User.id)]
+        )
+
+        sess = Session()
+        self.assert_compile(
+            sess.query(User).having(User.address_count > 5),
+            'SELECT (SELECT count(address.id) AS '
+            'count_1 FROM address WHERE address.user_id = "user".id) '
+            'AS anon_1, "user".id AS user_id FROM "user" '
+            'HAVING (SELECT count(address.id) AS '
+            'count_1 FROM address WHERE address.user_id = "user".id) '
+            '> :param_1'
+        )
+
+
+class AbstractTest(DeclarativeTestBase):
+
+    def test_abstract_boolean(self):
+
+        class A(Base):
+            __abstract__ = True
+            __tablename__ = 'x'
+            id = Column(Integer, primary_key=True)
+
+        class B(Base):
+            __abstract__ = False
+            __tablename__ = 'y'
+            id = Column(Integer, primary_key=True)
+
+        class C(Base):
+            __abstract__ = False
+            __tablename__ = 'z'
+            id = Column(Integer, primary_key=True)
+
+        class D(Base):
+            __tablename__ = 'q'
+            id = Column(Integer, primary_key=True)
+
+        eq_(set(Base.metadata.tables), set(['y', 'z', 'q']))
+
+    def test_middle_abstract_attributes(self):
+        # test for [ticket:3219]
+        class A(Base):
+            __tablename__ = 'a'
+
+            id = Column(Integer, primary_key=True)
+            name = Column(String)
+
+        class B(A):
+            __abstract__ = True
+            data = Column(String)
+
+        class C(B):
+            c_value = Column(String)
+
+        eq_(
+            sa.inspect(C).attrs.keys(), ['id', 'name', 'data', 'c_value']
+        )
+
+    def test_middle_abstract_inherits(self):
+        # test for [ticket:3240]
+
+        class A(Base):
+            __tablename__ = 'a'
+            id = Column(Integer, primary_key=True)
+
+        class AAbs(A):
+            __abstract__ = True
+
+        class B1(A):
+            __tablename__ = 'b1'
+            id = Column(ForeignKey('a.id'), primary_key=True)
+
+        class B2(AAbs):
+            __tablename__ = 'b2'
+            id = Column(ForeignKey('a.id'), primary_key=True)
+
+        assert B1.__mapper__.inherits is A.__mapper__
+
+        assert B2.__mapper__.inherits is A.__mapper__
