@@ -125,9 +125,11 @@ class MyClass(object):
 class UserDefinedExtensionTest(fixtures.ORMTest):
     @classmethod
     def teardown_class(cls):
-        clear_mappers()
         instrumentation._reinstall_default_lookups()
 
+    def teardown(self):
+        clear_mappers()
+
     def test_instance_dict(self):
         class User(MyClass):
             pass
@@ -483,5 +485,5 @@ class ExtendedEventsTest(fixtures.ORMTest):
 
         register_class(A)
         manager = instrumentation.manager_of_class(A)
-        assert issubclass(manager.dispatch._parent_cls.__dict__['dispatch'].events, MyEvents)
+        assert issubclass(manager.dispatch._events, MyEvents)
 
