@@ -5,12 +5,13 @@ from sqlalchemy import Integer, String
 from sqlalchemy.testing.schema import Table, Column
 from sqlalchemy.orm import mapper, relationship, \
     create_session, class_mapper, \
-    Mapper, column_property, \
+    Mapper, column_property, query, \
     Session, sessionmaker, attributes, configure_mappers
 from sqlalchemy.orm.instrumentation import ClassManager
 from sqlalchemy.orm import instrumentation, events
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing.util import gc_collect
 from test.orm import _fixtures
 from sqlalchemy import event
@@ -22,6 +23,7 @@ class _RemoveListeners(object):
         events.InstanceEvents._clear()
         events.SessionEvents._clear()
         events.InstrumentationEvents._clear()
+        events.QueryEvents._clear()
         super(_RemoveListeners, self).teardown()
 
 
@@ -41,7 +43,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             pass
 
         mapper(A, users)
-        mapper(B, addresses, inherits=A)
+        mapper(B, addresses, inherits=A,
+            properties={'address_id': addresses.c.id})
 
         def init_a(target, args, kwargs):
             canary.append(('init_a', target))
@@ -84,10 +87,6 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         for meth in [
             'init',
             'init_failure',
-            'translate_row',
-            'create_instance',
-            'append_result',
-            'populate_instance',
             'load',
             'refresh',
             'expire',
@@ -115,6 +114,7 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         mapper(User, users)
         canary = self.listen_all(User)
+        named_canary = self.listen_all(User, named=True)
 
         sess = create_session()
         u = User(name='u1')
@@ -128,14 +128,15 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess.flush()
         sess.delete(u)
         sess.flush()
-        eq_(canary,
-            ['init', 'before_insert',
-             'after_insert', 'expire', 'translate_row',
-             'populate_instance', 'refresh',
-             'append_result', 'translate_row', 'create_instance',
-             'populate_instance', 'load', 'append_result',
-             'before_update', 'after_update', 'before_delete',
-             'after_delete'])
+        expected = [
+            'init', 'before_insert',
+            'after_insert', 'expire',
+            'refresh',
+            'load',
+            'before_update', 'after_update', 'before_delete',
+            'after_delete']
+        eq_(canary, expected)
+        eq_(named_canary, expected)
 
     def test_insert_before_configured(self):
         users, User = self.tables.users, self.classes.User
@@ -220,7 +221,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             pass
 
         mapper(User, users)
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
 
         canary1 = self.listen_all(User, propagate=True)
         canary2 = self.listen_all(User)
@@ -238,16 +240,13 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess.delete(am)
         sess.flush()
         eq_(canary1, ['init', 'before_insert', 'after_insert',
-            'translate_row', 'populate_instance','refresh',
-            'append_result', 'translate_row', 'create_instance'
-            , 'populate_instance', 'load', 'append_result',
+            'refresh', 'load',
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
         eq_(canary2, [])
         eq_(canary3, ['init', 'before_insert', 'after_insert',
-            'translate_row', 'populate_instance','refresh',
-            'append_result', 'translate_row', 'create_instance'
-            , 'populate_instance', 'load', 'append_result',
+            'refresh',
+            'load',
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
 
@@ -264,7 +263,8 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
 
         class AdminUser(User):
             pass
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
         canary3 = self.listen_all(AdminUser)
 
         sess = create_session()
@@ -279,16 +279,12 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         sess.delete(am)
         sess.flush()
         eq_(canary1, ['init', 'before_insert', 'after_insert',
-            'translate_row', 'populate_instance','refresh',
-            'append_result', 'translate_row', 'create_instance'
-            , 'populate_instance', 'load', 'append_result',
+            'refresh', 'load',
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
         eq_(canary2, [])
         eq_(canary3, ['init', 'before_insert', 'after_insert',
-            'translate_row', 'populate_instance','refresh',
-            'append_result', 'translate_row', 'create_instance'
-            , 'populate_instance', 'load', 'append_result',
+            'refresh', 'load',
             'before_update', 'after_update', 'before_delete',
             'after_delete'])
 
@@ -372,25 +368,6 @@ class MapperEventsTest(_RemoveListeners, _fixtures.FixtureTest):
         eq_(m1.mock_calls, [call()])
         eq_(m2.mock_calls, [call()])
 
-    def test_retval(self):
-        User, users = self.classes.User, self.tables.users
-
-        def create_instance(mapper, context, row, class_):
-            u = User.__new__(User)
-            u.foo = True
-            return u
-
-        mapper(User, users)
-        event.listen(User, 'create_instance', create_instance, retval=True)
-        sess = create_session()
-        u1 = User()
-        u1.name = 'ed'
-        sess.add(u1)
-        sess.flush()
-        sess.expunge_all()
-        u = sess.query(User).first()
-        assert u.foo
-
     def test_instrument_event(self):
         Address, addresses, users, User = (self.classes.Address,
                                 self.tables.addresses,
@@ -853,7 +830,8 @@ class RemovalTest(_fixtures.FixtureTest):
             pass
 
         mapper(User, users)
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
 
         fn = Mock()
         event.listen(User.name, "set", fn, propagate=True)
@@ -960,6 +938,50 @@ class RefreshTest(_fixtures.FixtureTest):
         sess.query(User).first()
         eq_(canary, [])
 
+    def test_changes_reset(self):
+        """test the contract of load/refresh such that history is reset.
+
+        This has never been an official contract but we are testing it
+        here to ensure it is maintained given the loading performance
+        enhancements.
+
+        """
+        User = self.classes.User
+
+        @event.listens_for(User, "load")
+        def canary1(obj, context):
+            obj.name = 'new name!'
+
+        @event.listens_for(User, "refresh")
+        def canary2(obj, context, props):
+            obj.name = 'refreshed name!'
+
+        sess = Session()
+        u1 = User(name='u1')
+        sess.add(u1)
+        sess.commit()
+        sess.close()
+
+        u1 = sess.query(User).first()
+        eq_(
+            attributes.get_history(u1, "name"),
+            ((), ['new name!'], ())
+        )
+        assert "name" not in attributes.instance_state(u1).committed_state
+        assert u1 not in sess.dirty
+
+        sess.expire(u1)
+        u1.id
+        eq_(
+            attributes.get_history(u1, "name"),
+            ((), ['refreshed name!'], ())
+        )
+        assert "name" not in attributes.instance_state(u1).committed_state
+        assert u1 in sess.dirty
+
+
+
+
     def test_repeated_rows(self):
         User = self.classes.User
 
@@ -1176,6 +1198,7 @@ class SessionEventsTest(_RemoveListeners, _fixtures.FixtureTest):
             'before_commit', 'after_commit','after_transaction_end']
         )
 
+
     def test_rollback_hook(self):
         User, users = self.classes.User, self.tables.users
         sess, canary = self._listener_fixture()
@@ -1511,26 +1534,10 @@ class MapperExtensionTest(_fixtures.FixtureTest):
                 methods.append('init_failed')
                 return sa.orm.EXT_CONTINUE
 
-            def translate_row(self, mapper, context, row):
-                methods.append('translate_row')
-                return sa.orm.EXT_CONTINUE
-
-            def create_instance(self, mapper, selectcontext, row, class_):
-                methods.append('create_instance')
-                return sa.orm.EXT_CONTINUE
-
             def reconstruct_instance(self, mapper, instance):
                 methods.append('reconstruct_instance')
                 return sa.orm.EXT_CONTINUE
 
-            def append_result(self, mapper, selectcontext, row, instance, result, **flags):
-                methods.append('append_result')
-                return sa.orm.EXT_CONTINUE
-
-            def populate_instance(self, mapper, selectcontext, row, instance, **flags):
-                methods.append('populate_instance')
-                return sa.orm.EXT_CONTINUE
-
             def before_insert(self, mapper, connection, instance):
                 methods.append('before_insert')
                 return sa.orm.EXT_CONTINUE
@@ -1578,9 +1585,8 @@ class MapperExtensionTest(_fixtures.FixtureTest):
         sess.flush()
         eq_(methods,
             ['instrument_class', 'init_instance', 'before_insert',
-             'after_insert', 'translate_row', 'populate_instance',
-             'append_result', 'translate_row', 'create_instance',
-             'populate_instance', 'reconstruct_instance', 'append_result',
+             'after_insert',
+             'reconstruct_instance',
              'before_update', 'after_update', 'before_delete', 'after_delete'])
 
     def test_inheritance(self):
@@ -1594,7 +1600,8 @@ class MapperExtensionTest(_fixtures.FixtureTest):
             pass
 
         mapper(User, users, extension=Ext())
-        mapper(AdminUser, addresses, inherits=User)
+        mapper(AdminUser, addresses, inherits=User,
+            properties={'address_id': addresses.c.id})
 
         sess = create_session()
         am = AdminUser(name='au1', email_address='au1@e1')
@@ -1609,10 +1616,9 @@ class MapperExtensionTest(_fixtures.FixtureTest):
         sess.flush()
         eq_(methods,
             ['instrument_class', 'instrument_class', 'init_instance',
-             'before_insert', 'after_insert', 'translate_row',
-             'populate_instance', 'append_result', 'translate_row',
-             'create_instance', 'populate_instance', 'reconstruct_instance',
-             'append_result', 'before_update', 'after_update', 'before_delete',
+             'before_insert', 'after_insert',
+             'reconstruct_instance',
+             'before_update', 'after_update', 'before_delete',
              'after_delete'])
 
     def test_before_after_only_collection(self):
@@ -1670,7 +1676,8 @@ class MapperExtensionTest(_fixtures.FixtureTest):
 
         ext = Ext()
         mapper(User, users, extension=ext)
-        mapper(AdminUser, addresses, inherits=User, extension=ext)
+        mapper(AdminUser, addresses, inherits=User, extension=ext,
+            properties={'address_id': addresses.c.id})
 
         sess = create_session()
         am = AdminUser(name="au1", email_address="au1@e1")
@@ -1685,27 +1692,11 @@ class MapperExtensionTest(_fixtures.FixtureTest):
         sess.flush()
         eq_(methods,
             ['instrument_class', 'instrument_class', 'init_instance',
-             'before_insert', 'after_insert', 'translate_row',
-             'populate_instance', 'append_result', 'translate_row',
-             'create_instance', 'populate_instance', 'reconstruct_instance',
-             'append_result', 'before_update', 'after_update', 'before_delete',
+             'before_insert', 'after_insert',
+             'reconstruct_instance',
+             'before_update', 'after_update', 'before_delete',
              'after_delete'])
 
-    def test_create_instance(self):
-        User, users = self.classes.User, self.tables.users
-
-        class CreateUserExt(sa.orm.MapperExtension):
-            def create_instance(self, mapper, selectcontext, row, class_):
-                return User.__new__(User)
-
-        mapper(User, users, extension=CreateUserExt())
-        sess = create_session()
-        u1 = User()
-        u1.name = 'ed'
-        sess.add(u1)
-        sess.flush()
-        sess.expunge_all()
-        assert sess.query(User).first()
 
     def test_unnecessary_methods_not_evented(self):
         users = self.tables.users
@@ -1892,3 +1883,37 @@ class SessionExtensionTest(_fixtures.FixtureTest):
         assert not s.dispatch.after_commit
         assert len(s.dispatch.before_commit) == 1
 
+
+class QueryEventsTest(
+        _RemoveListeners, _fixtures.FixtureTest, AssertsCompiledSQL):
+    run_inserts = None
+    __dialect__ = 'default'
+
+    @classmethod
+    def setup_mappers(cls):
+        User = cls.classes.User
+        users = cls.tables.users
+
+        mapper(User, users)
+
+    def test_before_compile(self):
+        @event.listens_for(query.Query, "before_compile", retval=True)
+        def no_deleted(query):
+            for desc in query.column_descriptions:
+                if desc['type'] is User:
+                    entity = desc['expr']
+                    query = query.filter(entity.id != 10)
+            return query
+
+        User = self.classes.User
+        s = Session()
+
+        q = s.query(User).filter_by(id=7)
+        self.assert_compile(
+            q,
+            "SELECT users.id AS users_id, users.name AS users_name "
+            "FROM users "
+            "WHERE users.id = :id_1 AND users.id != :id_2",
+            checkparams={'id_2': 10, 'id_1': 7}
+        )
+
