@@ -61,11 +61,8 @@ class UnicodeTest(fixtures.MappedTest):
 
     @classmethod
     def define_tables(cls, metadata):
-        if testing.against('mysql+oursql'):
-            from sqlalchemy.dialects.mysql import VARCHAR
-            uni_type = VARCHAR(50, collation='utf8_unicode_ci')
-        else:
-            uni_type = sa.Unicode(50)
+        uni_type = sa.Unicode(50).with_variant(
+            sa.Unicode(50, collation="utf8_unicode_ci"), "mysql")
 
         Table('uni_t1', metadata,
             Column('id',  Integer, primary_key=True,
@@ -1129,11 +1126,12 @@ class OneToManyTest(_fixtures.FixtureTest):
 
             ("UPDATE addresses SET user_id=:user_id "
              "WHERE addresses.id = :addresses_id",
-             {'user_id': None, 'addresses_id': a1.id}),
+             [
+                {'user_id': None, 'addresses_id': a1.id},
+                {'user_id': u1.id, 'addresses_id': a3.id}
+            ]),
 
-            ("UPDATE addresses SET user_id=:user_id "
-             "WHERE addresses.id = :addresses_id",
-             {'user_id': u1.id, 'addresses_id': a3.id})])
+            ])
 
     def test_child_move(self):
         """Moving a child from one parent to another, with a delete.
@@ -1310,7 +1308,7 @@ class SaveTest(_fixtures.FixtureTest):
         session.add_all((u, u2))
         session.flush()
 
-        # assert the first one retreives the same from the identity map
+        # assert the first one retrieves the same from the identity map
         nu = session.query(m).get(u.id)
         assert u is nu
 
@@ -1408,7 +1406,9 @@ class SaveTest(_fixtures.FixtureTest):
 
         # define a mapper for AddressUser that inherits the User.mapper, and
         # joins on the id column
-        mapper(AddressUser, addresses, inherits=m1)
+        mapper(AddressUser, addresses, inherits=m1, properties={
+                'address_id': addresses.c.id
+            })
 
         au = AddressUser(name='u', email_address='u@e')
 
@@ -1430,7 +1430,7 @@ class SaveTest(_fixtures.FixtureTest):
         mapper(Order, orders, properties={
             'description': sa.orm.deferred(orders.c.description)})
 
-        # dont set deferred attribute, commit session
+        # don't set deferred attribute, commit session
         o = Order(id=42)
         session = create_session(autocommit=False)
         session.add(o)
@@ -2344,12 +2344,12 @@ class InheritingRowSwitchTest(fixtures.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         Table('parent', metadata,
-            Column('id', Integer, primary_key=True),
+            Column('pid', Integer, primary_key=True),
             Column('pdata', String(30))
         )
         Table('child', metadata,
-            Column('id', Integer, primary_key=True),
-            Column('pid', Integer, ForeignKey('parent.id')),
+            Column('cid', Integer, primary_key=True),
+            Column('pid', Integer, ForeignKey('parent.pid')),
             Column('cdata', String(30))
         )
 
@@ -2371,27 +2371,27 @@ class InheritingRowSwitchTest(fixtures.MappedTest):
         mapper(C, child, inherits=P)
 
         sess = create_session()
-        c1 = C(id=1, pdata='c1', cdata='c1')
+        c1 = C(pid=1, cid=1, pdata='c1', cdata='c1')
         sess.add(c1)
         sess.flush()
 
         # establish a row switch between c1 and c2.
         # c2 has no value for the "child" table
-        c2 = C(id=1, pdata='c2')
+        c2 = C(pid=1, cid=1, pdata='c2')
         sess.add(c2)
         sess.delete(c1)
 
         self.assert_sql_execution(testing.db, sess.flush,
-            CompiledSQL("UPDATE parent SET pdata=:pdata WHERE parent.id = :parent_id",
-                {'pdata':'c2', 'parent_id':1}
+            CompiledSQL("UPDATE parent SET pdata=:pdata WHERE parent.pid = :parent_pid",
+                {'pdata':'c2', 'parent_pid':1}
             ),
 
             # this fires as of [ticket:1362], since we synchronzize
             # PK/FKs on UPDATES.  c2 is new so the history shows up as
             # pure added, update occurs.  If a future change limits the
             # sync operation during _save_obj().update, this is safe to remove again.
-            CompiledSQL("UPDATE child SET pid=:pid WHERE child.id = :child_id",
-                {'pid':1, 'child_id':1}
+            CompiledSQL("UPDATE child SET pid=:pid WHERE child.cid = :child_cid",
+                {'pid':1, 'child_cid':1}
             )
         )
 
@@ -2479,7 +2479,8 @@ class PartialNullPKTest(fixtures.MappedTest):
         t1.col2 = 5
         assert_raises_message(
             orm_exc.FlushError,
-            "Can't update table using NULL for primary key value",
+            "Can't update table t1 using NULL for primary "
+            "key value on column t1.col2",
             s.commit
         )
 
@@ -2492,7 +2493,8 @@ class PartialNullPKTest(fixtures.MappedTest):
         t1.col3 = 'hi'
         assert_raises_message(
             orm_exc.FlushError,
-            "Can't update table using NULL for primary key value",
+            "Can't update table t1 using NULL for primary "
+            "key value on column t1.col2",
             s.commit
         )
 
@@ -2505,7 +2507,8 @@ class PartialNullPKTest(fixtures.MappedTest):
         s.delete(t1)
         assert_raises_message(
             orm_exc.FlushError,
-            "Can't delete from table using NULL for primary key value",
+            "Can't delete from table t1 using NULL "
+            "for primary key value on column t1.col2",
             s.commit
         )
 
