@@ -18,7 +18,8 @@ from sqlalchemy.testing.exclusions import \
      succeeds_if,\
      SpecPredicate,\
      against,\
-     LambdaPredicate
+     LambdaPredicate,\
+     requires_tag
 
 def no_support(db, reason):
     return SpecPredicate(db, description=reason)
@@ -29,7 +30,7 @@ def exclude(db, op, spec, description=None):
 class DefaultRequirements(SuiteRequirements):
     @property
     def deferrable_or_no_constraints(self):
-        """Target database must support derferable constraints."""
+        """Target database must support deferrable constraints."""
 
         return skip_if([
             no_support('firebird', 'not supported by database'),
@@ -37,6 +38,12 @@ class DefaultRequirements(SuiteRequirements):
             no_support('mssql', 'not supported by database'),
             ])
 
+    @property
+    def check_constraints(self):
+        """Target database must support check constraints."""
+
+        return exclusions.open()
+
     @property
     def named_constraints(self):
         """target database must support names for constraints."""
@@ -60,7 +67,7 @@ class DefaultRequirements(SuiteRequirements):
 
         return skip_if(
                     ['sqlite', 'oracle'],
-                    'target backend does not support ON UPDATE CASCADE'
+                    'target backend %(doesnt_support)s ON UPDATE CASCADE'
                 )
 
     @property
@@ -68,7 +75,8 @@ class DefaultRequirements(SuiteRequirements):
         """target database must *not* support ON UPDATE..CASCADE behavior in
         foreign keys."""
 
-        return fails_on_everything_except('sqlite', 'oracle', '+zxjdbc') + skip_if('mssql')
+        return fails_on_everything_except('sqlite', 'oracle', '+zxjdbc') + \
+            skip_if('mssql')
 
     @property
     def deferrable_fks(self):
@@ -118,6 +126,17 @@ class DefaultRequirements(SuiteRequirements):
                 "not supported by database"
             )
 
+    @property
+    def temporary_tables(self):
+        """target database supports temporary tables"""
+        return skip_if(
+                    ["mssql"], "sql server has some other syntax?"
+                )
+
+    @property
+    def temp_table_reflection(self):
+        return self.temporary_tables
+
     @property
     def reflectable_autoincrement(self):
         """Target database must support tables that can automatically generate
@@ -208,7 +227,7 @@ class DefaultRequirements(SuiteRequirements):
         return only_on(
                     ('postgresql', 'sqlite', 'mysql'),
                     "DBAPI has no isolation level support"
-                ).fails_on('postgresql+pypostgresql',
+                ) + fails_on('postgresql+pypostgresql',
                           'pypostgresql bombs on multiple isolation level calls')
 
     @property
@@ -294,6 +313,17 @@ class DefaultRequirements(SuiteRequirements):
                     "sqlite"
                 )
 
+    @property
+    def temp_table_names(self):
+        """target dialect supports listing of temporary table names"""
+
+        return only_on(['sqlite', 'oracle'])
+
+    @property
+    def temporary_views(self):
+        """target database supports temporary views"""
+        return only_on(['sqlite', 'postgresql'])
+
     @property
     def update_nowait(self):
         """Target database must support SELECT...FOR UPDATE NOWAIT"""
@@ -342,7 +372,7 @@ class DefaultRequirements(SuiteRequirements):
     @property
     def window_functions(self):
         return only_if([
-                    "postgresql", "mssql", "oracle"
+                    "postgresql>=8.4", "mssql", "oracle"
                 ], "Backend does not support window functions")
 
     @property
@@ -402,13 +432,25 @@ class DefaultRequirements(SuiteRequirements):
     def unicode_ddl(self):
         """Target driver must support some degree of non-ascii symbol names."""
         # TODO: expand to exclude MySQLdb versions w/ broken unicode
+
         return skip_if([
             no_support('oracle', 'FIXME: no support in database?'),
             no_support('sybase', 'FIXME: guessing, needs confirmation'),
             no_support('mssql+pymssql', 'no FreeTDS support'),
-
+            LambdaPredicate(
+                lambda config: against(config, "mysql+mysqlconnector") and
+                config.db.dialect._mysqlconnector_version_info > (2, 0) and
+                util.py2k,
+                "bug in mysqlconnector 2.0"
+            ),
+            LambdaPredicate(
+                lambda config: against(config, 'mssql+pyodbc') and
+                config.db.dialect.freetds and
+                config.db.dialect.freetds_driver_version < "0.91",
+                "older freetds doesn't support unicode DDL"
+            ),
             exclude('mysql', '<', (4, 1, 1), 'no unicode connection support'),
-            ])
+        ])
 
     @property
     def sane_rowcount(self):
@@ -418,13 +460,14 @@ class DefaultRequirements(SuiteRequirements):
         )
 
 
+
     @property
     def emulated_lastrowid(self):
         """"target dialect retrieves cursor.lastrowid or an equivalent
         after an insert() construct executes.
         """
         return fails_on_everything_except('mysql',
-                                      'sqlite+pysqlite',
+                                      'sqlite+pysqlite', 'sqlite+pysqlcipher',
                                       'sybase', 'mssql')
 
     @property
@@ -441,14 +484,14 @@ class DefaultRequirements(SuiteRequirements):
         """
         return skip_if('mssql+pymssql', 'crashes on pymssql') + \
                     fails_on_everything_except('mysql',
-                                       'sqlite+pysqlite')
+                                       'sqlite+pysqlite', 'sqlite+pysqlcipher')
 
     @property
     def sane_multi_rowcount(self):
         return fails_if(
-                    lambda config: not config.db.dialect.supports_sane_multi_rowcount,
-                    "driver doesn't support 'sane' multi row count"
-                )
+            lambda config: not config.db.dialect.supports_sane_multi_rowcount,
+            "driver %(driver)s %(doesnt_support)s 'sane' multi row count"
+        )
 
     @property
     def nullsordering(self):
@@ -613,6 +656,10 @@ class DefaultRequirements(SuiteRequirements):
                         'postgresql+pg8000', None, None,
                         'postgresql+pg8000 has FP inaccuracy even with '
                         'only four decimal places '),
+                    (
+                        'postgresql+psycopg2cffi', None, None,
+                        'postgresql+psycopg2cffi has FP inaccuracy even with '
+                        'only four decimal places '),
                 ])
 
     @property
@@ -697,15 +744,37 @@ class DefaultRequirements(SuiteRequirements):
                     "oracle_db_link option not specified in config"
                 )
 
+    @property
+    def postgresql_test_dblink(self):
+        return skip_if(
+                    lambda config: not config.file_config.has_option(
+                        'sqla_testing', 'postgres_test_db_link'),
+                    "postgres_test_db_link option not specified in config"
+                )
+
+    @property
+    def postgresql_jsonb(self):
+        return skip_if(
+            lambda config:
+            config.db.dialect.driver == "pg8000" and
+            config.db.dialect._dbapi_version <= (1, 10, 1)
+        )
+
     @property
     def percent_schema_names(self):
         return skip_if(
-                [
-                    ("+psycopg2", None, None,
-                            "psycopg2 2.4 no longer accepts % in bind placeholders"),
-                    ("mysql", None, None, "executemany() doesn't work here")
-                ]
-            )
+            [
+                (
+                    "+psycopg2", None, None,
+                    "psycopg2 2.4 no longer accepts percent "
+                    "sign in bind placeholders"),
+                (
+                    "+psycopg2cffi", None, None,
+                    "psycopg2cffi does not accept percent signs in "
+                    "bind placeholders"),
+                ("mysql", None, None, "executemany() doesn't work here")
+            ]
+        )
 
     @property
     def order_by_label_with_expression(self):
@@ -726,16 +795,16 @@ class DefaultRequirements(SuiteRequirements):
             )
 
     @property
-    def threading_with_mock(self):
-        """Mark tests that use threading and mock at the same time - stability
-        issues have been observed with coverage + python 3.3
-
-        """
-        return skip_if(
-                lambda config: util.py3k and
-                    config.options.has_coverage,
-                "Stability issues with coverage + py3k"
+    def mssql_freetds(self):
+        return only_on(
+            LambdaPredicate(
+                lambda config: (
+                    (against(config, 'mssql+pyodbc') and
+                     config.db.dialect.freetds)
+                    or against(config, 'mssql+pymssql')
+                )
             )
+        )
 
     @property
     def selectone(self):
@@ -755,3 +824,9 @@ class DefaultRequirements(SuiteRequirements):
         return against(config, 'mysql') and \
                 config.db.dialect._detect_casing(config.db) == 0
 
+    @property
+    def postgresql_utf8_server_encoding(self):
+        return only_if(
+            lambda config: against(config, 'postgresql') and
+            config.db.scalar("show server_encoding").lower() == "utf8"
+        )
